/* Makes the system report from the reports
   made by the monitor plugins. 
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include <sys/types.h>
#include <sys/stat.h>
 
#include "monitor.h"
#include "options.h"
#include "report.h"
#include "version.h"
#include "modulefunc.h"
#include "logfunc.h"
#include "loop.h"

char *sysreport_file;

/* Creates the system report. */
void createsysreport(){
  int i;
  FILE *sysreport_fd;

  /* Open the sysreport file. */
  sysreport_file = mktempname();
  
  sysreport_fd = fopen(sysreport_file,"a");

  /* Change the permissions of the sysreport file so
     only the we can read/write to it. */
  chmod(sysreport_file,S_IREAD | S_IWRITE);

  /* Create the header. */
  fprintf(sysreport_fd,"BEGINREPORT %s\n",senderversion);

  /* Copy the internal errors. */
  if (logfile){
    addfile(sysreport_fd,logfile);
    rmtempfile(logfile);
    logfile = NULL;
  }

  /* Go through each monitor and create the report. */
  if (!just_dmons){
    for (i = 0; i < nummonitors; i++){

      /* Print the name of the monitor. */
      fprintf(sysreport_fd,"MONITOR %s\n",monitorlist[i].title);

      /* Call addfile to add the stdout file to sysreport_fd. */
      addfile(sysreport_fd,monitorlist[i].stdout_file);

      if (!nodelete){
	/* Delete the stdout and stdin files. */
	rmtempfile(monitorlist[i].stdout_file);
	rmtempfile(monitorlist[i].stdin_file);
      }

      /* Include a beginning newline in case the monitor didn't
	 have a newline at the end. */
      fprintf(sysreport_fd,"\nDONEMONITOR\n");
    }
  }

  /* Go through each dmonitor and create the report. */
  for (i = 0; i < numdmonitors; i++){

    /* Print the name of the monitor. */
    fprintf(sysreport_fd,"MONITOR %s\n",dmonitorlist[i].title);

    /* Call addfile to add the report file to sysreport_fd. */
    addfile(sysreport_fd,dmonitorlist[i].report_file);

    if (!nodelete){
      /* Delete the report file. */
      rmtempfile(dmonitorlist[i].report_file);
    }

    /* Include a beginning newline in case the dmonitor didn't
       have a newline at the end. */
    fprintf(sysreport_fd,"\nDONEMONITOR\n");
  }

  fprintf(sysreport_fd,"DONEREPORT\n");

  fclose(sysreport_fd);
}

/* Add a source file to a target fd. */
void addfile(FILE *target,char *sourcename){
  FILE *source;
  char tmpchar;
  int r;

  source = fopen(sourcename,"r");

  while (1){

    r = fscanf(source,"%c",&tmpchar);

    if (r == EOF){
      fclose(source);
      return;
    }


    /* Before we print the character check if it really is
       a valid character for security reasons. */
    if (!(isprint(tmpchar) | isspace(tmpchar)))
      tmpchar = ' ';

    fprintf(target,"%c",tmpchar);
  }
}


static char *id="@(#) $Id: report.c,v 4.0 1999/10/07 22:56:15 pete Exp $";

/* 
 * $Log: report.c,v $
 * Revision 4.0  1999/10/07 22:56:15  pete
 * Started work on version 1.0
 *
 * Revision 3.2  1999/09/19 18:25:56  pete
 * Added support for automake
 *
 * Revision 3.1  1999/09/19 14:36:40  pete
 * Fixed a bug related to SIGUSR1. Now if just_dmons is set createsysreport()
 * won't try to get reports from the normal monitors.
 *
 * Revision 3.0  1999/09/19 01:15:24  pete
 * Started work on 0.03
 *
 * Revision 2.2  1999/09/17 22:34:51  pete
 * Added support for logfunc()
 *
 * Revision 2.1  1999/09/17 20:38:39  pete
 * Added support for dmonitors. Support works perfectly! :)
 *
 * Revision 2.0  1999/09/15 16:05:52  pete
 * Started work on version 0.02
 *
 * Revision 1.15  1999/09/11 23:25:08  pete
 * Changed remove(sometempfile) to rmtempfile(sometempfile)
 *
 * Revision 1.14  1999/09/11 20:19:04  pete
 * *** empty log message ***
 *
 * Revision 1.13  1999/09/11 20:15:56  pete
 * Changed tempname to POSIX complient mktempname
 *
 * Revision 1.12  1999/09/11 19:22:59  pete
 * Added support for the nodelete option.
 *
 * Revision 1.11  1999/09/11 02:13:03  pete
 * Removed some debugging printf's
 *
 * Revision 1.10  1999/09/11 02:07:34  pete
 * Added ID and LOG tags after much work. :)
 *
 */
