/* Sender plugin functions. */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>
#include <unistd.h>

#include "modulefunc.h"
#include "report.h"
#include "sender.h"
#include "options.h"
#include "runcmd.h"
#include "version.h"
#include "logfunc.h"

SENDER *senderlist;
int numsenders;

int did_sender_succeed(int i);

/* Sends a system report. */
void sendsysreport(){
  int i,send_status;
  FILE *stdin_fd;

  for (i = 0; i < numsenders; i++){

    if (!daemon_mode)
      printf("Trying sender %s\n",senderlist[i].title);

    senderlist[i].stdin_file = mktempname();
    senderlist[i].stdout_file = mktempname();

    senderlist[i].pid = re_system(senderlist[i].cmd,senderlist[i].stdin_file,senderlist[i].stdout_file);

    /* Give the sender the report via standard input. */
    stdin_fd = fopen(senderlist[i].stdin_file,"a"); 
    addfile(stdin_fd,sysreport_file);
    fclose(stdin_fd);

    /* Make sure the sender exits. */
    wait_module(senderlist[i].pid,sendertimeout);

    send_status = did_sender_succeed(i);

    /* Did the sender fail? */
    if (send_status == -1)
      ; /* Do nothing. */
    else if (send_status == 1){
      asprintf(&errmsg,"Clear failure of sender %s\n",senderlist[i].title);
      logmsg(errmsg,ESM_LOG_ERR);
    }
    else{
      asprintf(&errmsg,"Unclear failure of sender %s - Sender did not return fail code.\n",senderlist[i].cmd);
      logmsg(errmsg,ESM_LOG_ERR);
    }

    if (!nodelete){
      /* Delete the stdin and stdout files. */
      rmtempfile(senderlist[i].stdin_file);
      rmtempfile(senderlist[i].stdout_file);
    }

    if (send_status == -1){
      /* Have we ran all of the senders in the priority group? */
      if (i == (numsenders - 1))
	break;
      else if (senderlist[i].pri != senderlist[i + 1].pri)
	break;
    }
  }

  if (!nodelete){
    /* Delete the report file. */
    rmtempfile(sysreport_file);
  }
}

/* Returns true if a sender succeeded. 1 if there is a clear failure, 0 if nothing was returned in
 standard output.
*/
int did_sender_succeed(int i){
  FILE *stdout_fd;
  char tmpbuff[1024];
  char *tmpbuff2;

  char strsucceed[] = "SUCCEED\n";
  char strfail[] = "FAIL\n";

  /* Open the standard output file. */
  stdout_fd = fopen(senderlist[i].stdout_file,"r");

  /* Read the first bit of the file and check for
     the string "SUCCEED\n" 
  */
  tmpbuff2 = fgets(tmpbuff,1024,stdout_fd);

  if (!daemon_mode && tmpbuff2)
    printf("Sender returned %s\n",tmpbuff);

  fclose(stdout_fd);

  if (strcmp(strsucceed,tmpbuff) == 0)
    return -1;
  else if (strcmp(strfail,tmpbuff) == 0)
    return 1;

  return 0;
}

/* Adds a sender of priority pri. Note that pri is currently ignored. */
void addsender(SENDER newsend,int pri){
  int i,newi;

  numsenders++;
  senderlist = realloc(senderlist,sizeof(SENDER) * numsenders);

  /* Only search for a spot if we have to. */
  if (numsenders > 2){

    /* Go through the sender list from the end to the start. If the priority of the sender is greater then
       pri move the sender +1 Otherwise take the above (+1) the spot we are looking at. Note that we
       start at the *second* last position. This way if the first position we check is less then out pri
       we can take the above (+1) position. 

       We also stop at -1 So when we find ourselves at the start we can take the first entry.
    */
    for (i = numsenders - 2; i >= -1; i--){
      if (i < 0){
	newi = 0;
	break;
      }
      else if (senderlist[i].pri > pri)
	senderlist[i + 1] = senderlist[i];
      else{
	newi = i + 1; /* Set newi to the position above this one. */
	break;
      }
    }
  }
  else if (numsenders == 2){
    if (pri > senderlist[0].pri){
      senderlist[1] = newsend;
      newi = 1;
    }
    else{
      senderlist[1] = senderlist[0];
      newi = 0;
    }
  }
  else
    newi = 0;

  /* Since the strings are just pointers to a string we have make a
     copy of the strings. Otherwise they might be overwritten later.
  */
  senderlist[newi].title = malloc(strlen(newsend.title) + 1);
  senderlist[newi].title[0] = '\000';
  senderlist[newi].title = strcpy(senderlist[newi].title,newsend.title);

  senderlist[newi].cmd = malloc(strlen(newsend.cmd) + 1);
  senderlist[newi].cmd[0] = '\000';
  senderlist[newi].cmd = strcpy(senderlist[newi].cmd,newsend.cmd);

  senderlist[newi].pri = pri;
}

static char *id="@(#) $Id: sender.c,v 4.0 1999/10/07 22:56:15 pete Exp $";

/* 
 * $Log: sender.c,v $
 * Revision 4.0  1999/10/07 22:56:15  pete
 * Started work on version 1.0
 *
 * Revision 3.3  1999/09/21 02:09:42  pete
 * *** empty log message ***
 *
 * Revision 3.2  1999/09/21 00:37:04  pete
 * Added support for priority groups.
 *
 * Revision 3.1  1999/09/19 18:25:57  pete
 * Added support for automake
 *
 * Revision 3.0  1999/09/19 01:15:25  pete
 * Started work on 0.03
 *
 * Revision 2.3  1999/09/17 22:49:20  pete
 * Added support/improved support for logmsg()
 *
 * Revision 2.2  1999/09/17 15:04:25  pete
 * Fixed some string allocation errors in addsender()
 *
 * Revision 2.1  1999/09/15 23:20:44  pete
 * *** empty log message ***
 *
 * Revision 2.0  1999/09/15 16:05:52  pete
 * Started work on version 0.02
 *
 * Revision 1.22  1999/09/12 20:46:04  pete
 * Made pri actually do something in addsender
 *
 * Revision 1.21  1999/09/12 19:44:11  pete
 * Fixed a bug where addsender and addmonitor were overwriting the title with
 * the cmd. Also changed some printf's
 *
 * Revision 1.20  1999/09/12 18:52:32  pete
 * *** empty log message ***
 *
 * Revision 1.19  1999/09/12 00:04:34  pete
 * Bug fix. Tried to free a string that was just freed by rmtempfile
 *
 * Revision 1.18  1999/09/11 23:25:08  pete
 * Changed remove(sometempfile) to rmtempfile(sometempfile)
 *
 * Revision 1.17  1999/09/11 20:21:37  pete
 * Fixed a bug caused by my fix of the 10 byte memory leak. (blah!)
 *
 * Revision 1.16  1999/09/11 20:17:35  pete
 * Fixed a 10 byte memory leak. :)
 *
 * Revision 1.15  1999/09/11 20:15:57  pete
 * Changed tempname to POSIX complient mktempname
 *
 * Revision 1.14  1999/09/11 19:51:35  pete
 * Made the "Unclear Failure" error message a bit more descriptive.
 *
 * Revision 1.13  1999/09/11 19:22:59  pete
 * Added support for the nodelete option.
 *
 * Revision 1.12  1999/09/11 02:13:03  pete
 * Removed some debugging printf's
 *
 * Revision 1.11  1999/09/11 02:07:34  pete
 * Added ID and LOG tags after much work. :)
 *
 */


